#!/usr/bin/env python

import sys
import os
import math
import numpy as np
from math import sqrt
from amplpy import AMPL, DataFrame
import time
import matplotlib.pyplot as plt

myZero = 1e-9
ShowPlots = False

## read problem data file written in AMPL .dat format
def readDat(filename):
    readparam = "[None]"
    m = 0
    n = 0
    dens = 1.0
    A = {}
    xhat = {}
    with open(filename) as f:
        for line in f:
            # look at file line by line
            line = line.strip()
            if len(line) == 0:
                continue
            if line[0] == '#':
                continue
            L = line.split()
            if L[0] == "param":
                readparam = L[1]
                if readparam == "m":
                    m = int(L[3][0:-1])
                elif readparam == "n":
                    n = int(L[3][0:-1])
                elif readparam == "dens":
                    dens = float(L[3][0:-1])
                continue
            elif L[0] == ';':
                readparam = "[None]"
                continue
            elif readparam == "xhat":
                j = int(L[0])
                xhatj = float(L[1])
                xhat[j] = xhatj
            elif readparam == "A":
                i = int(L[0])
                j = int(L[1])
                Aij = float(L[2])
                A[(i,j)] = Aij
    return (m,n,dens,A,xhat)

####################### MAIN #######################

## read command line
if len(sys.argv) < 2:
    print "syntax: " + sys.argv[0] + " Axhat.dat"
    exit(1)

t0 = time.time()
print "reading data in file", sys.argv[1]
(m,n,dens,Ad,xhatd) = readDat(sys.argv[1])
xhat = np.zeros(n)
for j in xhatd:
    xhat[j-1] = xhatd[j]
A = np.zeros((m,n))
for (i,j) in Ad:
    A[i-1,j-1] = Ad[(i,j)]

# check orthogonality of columns of A
print "checking orthogonality of A"
A = A / (n*dens) #m
AA = np.dot(A.T,A)
ell2err = np.linalg.norm(np.subtract(AA, np.eye(n))) / n**2
print "avg ||A'A - I_n||_2 = ", ell2err
ell2errdiag = np.linalg.norm(np.subtract(np.diag(AA),np.diag(np.eye(n)))) / n
print "avg ||diag(A'A) - diag(I_n)||_2 = ", ell2errdiag

ell1errdiag = 0
ell1erroffdiag = 0
for i in range(n-1):
    for j in range(i,n):
        AiAj = np.dot(A[:,i],A[:,j])
        if i == j:
            ell1errdiag += abs(AiAj-1)
        else:
            ell1erroffdiag += abs(AiAj)
        #print "<A_" + str(i) + "," + "A_" + str(j) + "> = " + str(AiAj/m)
ell1err = ell1errdiag + ell1erroffdiag
ell1err = ell1err / (n*(n+1)/2)
ell1errdiag = ell1errdiag / n
ell1erroffdiag = ell1erroffdiag / (n*(n-1)/2)
        
print "avg ||A'A - I_n||_1 = ", ell1err
print "avg ||diag(A'A) - diag(I_n)||_1 =", ell1errdiag
print "avg ||offd(A'A) - offd(I_n)||_1 =", ell1erroffdiag

print "ell2err =", ell2err, "; ell1err/sqrt(m) =", ell1err / sqrt(m)
print "ell2errdiag =", ell2errdiag, "; ell1errdiag/sqrt(m) =", ell1errdiag / sqrt(m)

# check eigenvalues of A
print "checking eigenvalues of A'A"
evals, evecs = np.linalg.eig(AA)
avgimagerr = sum(evals.imag)/n
print "avg imaginary parts =", avgimagerr
evals = [e for e in evals.real if e>myZero]
nevals = len(evals)
print "|nonzero eigenvalues| =", nevals
sevals = sorted(evals, reverse=True)
#print "3 largest eigvals: ", sevals[0:3]
#print "3 smallest eigvals:", sevals[-3:]
print "max eigenvalue =", sevals[0]
print "min eigenvalue =", sevals[-1]
delta = max(abs(1-sevals[0]), abs(1-sevals[-1]))
print "RIP delta =", delta

if ShowPlots:
    # plot eigenvalues
    x = range(nevals)
    plt.plot(x, evals, label='eigenvals')
    plt.plot(x, sevals, label='sorted')
    plt.xlabel('eig index')
    plt.ylabel('eig value')
    plt.title("Eigenvalues of A'A")
    plt.legend()
    plt.show()
    

